@@ -28,10 +28,10 @@ import socket
 import sys
 import time
 
-from ansible.errors import AnsibleOptionsError
 from ansible.cli import CLI
+from ansible.errors import AnsibleOptionsError
 from ansible.module_utils._text import to_native
-from ansible.plugins import module_loader
+from ansible.plugins.loader import module_loader
 from ansible.utils.cmd_functions import run_cmd
 
 try:
@@ -44,21 +44,48 @@ except ImportError:
 ########################################################
 
 class PullCLI(CLI):
-    ''' code behind ansible ad-hoc cli'''
+    ''' is used to up a remote copy of ansible on each managed node,
+        each set to run via cron and update playbook source via a source repository.
+        This inverts the default *push* architecture of ansible into a *pull* architecture,
+        which has near-limitless scaling potential.
+
+        The setup playbook can be tuned to change the cron frequency, logging locations, and parameters to ansible-pull.
+        This is useful both for extreme scale-out as well as periodic remediation.
+        Usage of the 'fetch' module to retrieve logs from ansible-pull runs would be an
+        excellent way to gather and analyze remote logs from ansible-pull.
+    '''
 
     DEFAULT_REPO_TYPE = 'git'
     DEFAULT_PLAYBOOK = 'local.yml'
     PLAYBOOK_ERRORS = {
         1: 'File does not exist',
-        2: 'File is not readable'
+        2: 'File is not readable',
     }
     SUPPORTED_REPO_MODULES = ['git']
+    ARGUMENTS = {'playbook.yml': 'The name of one the YAML format files to run as an Ansible playbook.'
+                                 'This can be a relative path within the checkout. By default, Ansible will'
+                                 "look for a playbook based on the host's fully-qualified domain name,"
+                                 'on the host hostname and finally a playbook named *local.yml*.', }
+
+    SKIP_INVENTORY_DEFAULTS = True
+
+    def _get_inv_cli(self):
+
+        inv_opts = ''
+        if getattr(self.options, 'inventory'):
+            for inv in self.options.inventory:
+                if isinstance(inv, list):
+                    inv_opts += " -i '%s' " % ','.join(inv)
+                elif ',' in inv or os.path.exists(inv):
+                    inv_opts += ' -i %s ' % inv
+
+        return inv_opts
 
     def parse(self):
         ''' create an options parser for bin/ansible '''
 
         self.parser = CLI.base_parser(
-            usage='%prog -U <repository> [options]',
+            usage='%prog -U <repository> [options] [<playbook.yml>]',
             connect_opts=True,
             vault_opts=True,
             runtask_opts=True,
@@ -66,40 +93,36 @@ class PullCLI(CLI):
             inventory_opts=True,
             module_opts=True,
             runas_prompt_opts=True,
+            desc="pulls playbooks from a VCS repo and executes them for the local host",
         )
 
         # options unique to pull
-        self.parser.add_option('--purge', default=False, action='store_true',
-            help='purge checkout after playbook run')
+        self.parser.add_option('--purge', default=False, action='store_true', help='purge checkout after playbook run')
         self.parser.add_option('-o', '--only-if-changed', dest='ifchanged', default=False, action='store_true',
-            help='only run the playbook if the repository has been updated')
+                               help='only run the playbook if the repository has been updated')
         self.parser.add_option('-s', '--sleep', dest='sleep', default=None,
-            help='sleep for random interval (between 0 and n number of seconds) before starting. This is a useful way to disperse git requests')
+                               help='sleep for random interval (between 0 and n number of seconds) before starting. '
+                                    'This is a useful way to disperse git requests')
         self.parser.add_option('-f', '--force', dest='force', default=False, action='store_true',
-            help='run the playbook even if the repository could not be updated')
-        self.parser.add_option('-d', '--directory', dest='dest', default=None,
-            help='directory to checkout repository to')
-        self.parser.add_option('-U', '--url', dest='url', default=None,
-            help='URL of the playbook repository')
-        self.parser.add_option('--full', dest='fullclone', action='store_true',
-            help='Do a full clone, instead of a shallow one.')
+                               help='run the playbook even if the repository could not be updated')
+        self.parser.add_option('-d', '--directory', dest='dest', default=None, help='directory to checkout repository to')
+        self.parser.add_option('-U', '--url', dest='url', default=None, help='URL of the playbook repository')
+        self.parser.add_option('--full', dest='fullclone', action='store_true', help='Do a full clone, instead of a shallow one.')
         self.parser.add_option('-C', '--checkout', dest='checkout',
-            help='branch/tag/commit to checkout.  ' 'Defaults to behavior of repository module.')
+                               help='branch/tag/commit to checkout. Defaults to behavior of repository module.')
         self.parser.add_option('--accept-host-key', default=False, dest='accept_host_key', action='store_true',
-            help='adds the hostkey for the repo url if not already added')
+                               help='adds the hostkey for the repo url if not already added')
         self.parser.add_option('-m', '--module-name', dest='module_name', default=self.DEFAULT_REPO_TYPE,
-            help='Repository module name, which ansible will use to check out the repo. Default is %s.' % self.DEFAULT_REPO_TYPE)
+                               help='Repository module name, which ansible will use to check out the repo. Default is %s.' % self.DEFAULT_REPO_TYPE)
         self.parser.add_option('--verify-commit', dest='verify', default=False, action='store_true',
-            help='verify GPG signature of checked out commit, if it fails abort running the playbook.'
-                 ' This needs the corresponding VCS module to support such an operation')
+                               help='verify GPG signature of checked out commit, if it fails abort running the playbook. '
+                                    'This needs the corresponding VCS module to support such an operation')
         self.parser.add_option('--clean', dest='clean', default=False, action='store_true',
-            help='modified files in the working repository will be discarded')
+                               help='modified files in the working repository will be discarded')
         self.parser.add_option('--track-subs', dest='tracksubs', default=False, action='store_true',
-            help='submodules will track the latest changes'
-                 ' This is equivalent to specifying the --remote flag to git submodule update')
-
-        # for pull we don't wan't a default
-        self.parser.set_defaults(inventory=None)
+                               help='submodules will track the latest changes. This is equivalent to specifying the --remote flag to git submodule update')
+        self.parser.add_option("--check", default=False, dest='check', action='store_true',
+                               help="don't make any changes; instead, try to predict some of the changes that may occur")
 
         super(PullCLI, self).parse()
 
@@ -109,9 +132,12 @@ class PullCLI(CLI):
             self.options.dest = os.path.join('~/.ansible/pull', hostname)
         self.options.dest = os.path.expandvars(os.path.expanduser(self.options.dest))
 
+        if os.path.exists(self.options.dest) and not os.path.isdir(self.options.dest):
+            raise AnsibleOptionsError("%s is not a valid or accessible directory." % self.options.dest)
+
         if self.options.sleep:
             try:
-                secs = random.randint(0,int(self.options.sleep))
+                secs = random.randint(0, int(self.options.sleep))
                 self.options.sleep = secs
             except ValueError:
                 raise AnsibleOptionsError("%s is not a number." % self.options.sleep)
@@ -120,7 +146,7 @@ class PullCLI(CLI):
             raise AnsibleOptionsError("URL for repository not specified, use -h for help")
 
         if self.options.module_name not in self.SUPPORTED_REPO_MODULES:
-            raise AnsibleOptionsError("Unsuported repo module %s, choices are %s" % (self.options.module_name, ','.join(self.SUPPORTED_REPO_MODULES)))
+            raise AnsibleOptionsError("Unsupported repo module %s, choices are %s" % (self.options.module_name, ','.join(self.SUPPORTED_REPO_MODULES)))
 
         display.verbosity = self.options.verbosity
         self.validate_conflicts(vault_opts=True)
@@ -142,16 +168,15 @@ class PullCLI(CLI):
         limit_opts = 'localhost,%s,127.0.0.1' % ','.join(set([host, node, host.split('.')[0], node.split('.')[0]]))
         base_opts = '-c local '
         if self.options.verbosity > 0:
-            base_opts += ' -%s' % ''.join([ "v" for x in range(0, self.options.verbosity) ])
+            base_opts += ' -%s' % ''.join(["v" for x in range(0, self.options.verbosity)])
 
         # Attempt to use the inventory passed in as an argument
         # It might not yet have been downloaded so use localhost as default
-        if not self.options.inventory or ( ',' not in self.options.inventory and not os.path.exists(self.options.inventory)):
-            inv_opts = 'localhost,'
-        else:
-            inv_opts = self.options.inventory
+        inv_opts = self._get_inv_cli()
+        if not inv_opts:
+            inv_opts = " -i localhost, "
 
-        #FIXME: enable more repo modules hg/svn?
+        # FIXME: enable more repo modules hg/svn?
         if self.options.module_name == 'git':
             repo_opts = "name=%s dest=%s" % (self.options.url, self.options.dest)
             if self.options.checkout:
@@ -181,7 +206,7 @@ class PullCLI(CLI):
 
         bin_path = os.path.dirname(os.path.abspath(sys.argv[0]))
         # hardcode local and inventory/host as this is just meant to fetch the repo
-        cmd = '%s/ansible -i "%s" %s -m %s -a "%s" all -l "%s"' % (bin_path, inv_opts, base_opts, self.options.module_name, repo_opts, limit_opts)
+        cmd = '%s/ansible %s %s -m %s -a "%s" all -l "%s"' % (bin_path, inv_opts, base_opts, self.options.module_name, repo_opts, limit_opts)
 
         for ev in self.options.extra_vars:
             cmd += ' -e "%s"' % ev
@@ -211,10 +236,10 @@ class PullCLI(CLI):
 
         # Build playbook command
         cmd = '%s/ansible-playbook %s %s' % (bin_path, base_opts, playbook)
-        if self.options.vault_password_file:
-            cmd += " --vault-password-file=%s" % self.options.vault_password_file
-        if self.options.inventory:
-            cmd += ' -i "%s"' % self.options.inventory
+        if self.options.vault_password_files:
+            for vault_password_file in self.options.vault_password_files:
+                cmd += " --vault-password-file=%s" % vault_password_file
+
         for ev in self.options.extra_vars:
             cmd += ' -e "%s"' % ev
         if self.options.ask_sudo_pass or self.options.ask_su_pass or self.options.become_ask_pass:
@@ -227,9 +252,16 @@ class PullCLI(CLI):
             cmd += ' -l "%s"' % self.options.subset
         else:
             cmd += ' -l "%s"' % limit_opts
+        if self.options.check:
+            cmd += ' -C'
 
         os.chdir(self.options.dest)
 
+        # redo inventory options as new files might exist now
+        inv_opts = self._get_inv_cli()
+        if inv_opts:
+            cmd += inv_opts
+
         # RUN THE PLAYBOOK COMMAND
         display.debug("running ansible-playbook to do actual work")
         display.debug('EXEC: %s' % cmd)
