@@ -1,25 +1,13 @@
 #!/usr/bin/python
-
 #
 # Copyright (c) 2015 CenturyLink
-#
-# This file is part of Ansible.
-#
-# Ansible is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# Ansible is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Ansible.  If not, see <http://www.gnu.org/licenses/>
-#
+# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
-ANSIBLE_METADATA = {'metadata_version': '1.0',
+from __future__ import absolute_import, division, print_function
+__metaclass__ = type
+
+
+ANSIBLE_METADATA = {'metadata_version': '1.1',
                     'status': ['preview'],
                     'supported_by': 'community'}
 
@@ -464,7 +452,9 @@ servers:
 
 __version__ = '${version}'
 
-from time import sleep
+import json
+import os
+import time
 from distutils.version import LooseVersion
 
 try:
@@ -488,6 +478,8 @@ except ImportError:
 else:
     CLC_FOUND = True
 
+from ansible.module_utils.basic import AnsibleModule
+
 
 class ClcServer:
     clc = clc_sdk
@@ -732,10 +724,8 @@ class ClcServer:
                 location = account.data.get('primaryDataCenter')
             data_center = clc.v2.Datacenter(location)
             return data_center
-        except CLCException as ex:
-            module.fail_json(
-                msg=str(
-                    "Unable to find location: {0}".format(location)))
+        except CLCException:
+            module.fail_json(msg="Unable to find location: {0}".format(location))
 
     @staticmethod
     def _find_alias(clc, module):
@@ -1565,7 +1555,7 @@ class ClcServer:
                 if retries == 0:
                     return module.fail_json(
                         msg='Unable to reach the CLC API after 5 attempts')
-                sleep(back_out)
+                time.sleep(back_out)
                 back_out *= 2
 
     @staticmethod
@@ -1588,6 +1578,6 @@ def main():
     clc_server = ClcServer(module)
     clc_server.process_request()
 
-from ansible.module_utils.basic import *  # pylint: disable=W0614
+
 if __name__ == '__main__':
     main()
