@@ -19,7 +19,7 @@
 # along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-ANSIBLE_METADATA = {'metadata_version': '1.0',
+ANSIBLE_METADATA = {'metadata_version': '1.1',
                     'status': ['preview'],
                     'supported_by': 'community'}
 
@@ -27,22 +27,23 @@ ANSIBLE_METADATA = {'metadata_version': '1.0',
 DOCUMENTATION = '''
 ---
 module: ovirt_hosts
-short_description: Module to manage hosts in oVirt
+short_description: Module to manage hosts in oVirt/RHV
 version_added: "2.3"
 author: "Ondra Machacek (@machacekondra)"
 description:
-    - "Module to manage hosts in oVirt"
+    - "Module to manage hosts in oVirt/RHV"
 options:
     name:
         description:
-            - "Name of the the host to manage."
+            - "Name of the host to manage."
         required: true
     state:
         description:
             - "State which should a host to be in after successful completion."
+            - "I(iscsilogin) and I(iscsidiscover) are supported since version 2.4."
         choices: [
             'present', 'absent', 'maintenance', 'upgraded', 'started',
-            'restarted', 'stopped', 'reinstalled'
+            'restarted', 'stopped', 'reinstalled', 'iscsidiscover', 'iscsilogin'
         ]
         default: present
     comment:
@@ -74,7 +75,7 @@ options:
     override_iptables:
         description:
             - "If True host iptables will be overridden by host deploy script."
-            - "Note that C(override_iptables) is I(false) by default in oVirt."
+            - "Note that C(override_iptables) is I(false) by default in oVirt/RHV."
     force:
         description:
             - "If True host will be forcibly moved to desired state."
@@ -103,6 +104,31 @@ options:
             - "If I(undeploy) it means this host should un-deploy hosted engine
                components and this host will not function as part of the High
                Availability cluster."
+    power_management_enabled:
+        description:
+            - "Enable or disable power management of the host."
+            - "For more comprehensive setup of PM use C(ovirt_host_pm) module."
+        version_added: 2.4
+    activate:
+        description:
+            - "If C(state) is I(present) activate the host."
+            - "This parameter is good to disable, when you don't want to change
+               the state of host when using I(present) C(state)."
+        default: True
+        version_added: 2.4
+    iscsi:
+        description:
+          - "If C(state) is I(iscsidiscover) it means that the iscsi attribute is being
+             used to discover targets"
+          - "If C(state) is I(iscsilogin) it means that the iscsi attribute is being
+             used to login to the specified targets passed as part of the iscsi attribute"
+        version_added: "2.4"
+    check_upgrade:
+        description:
+            - "If I(true) and C(state) is I(upgraded) run check for upgrade
+               action before executing upgrade action."
+        default: True
+        version_added: 2.4
 extends_documentation_fragment: ovirt
 '''
 
@@ -111,7 +137,7 @@ EXAMPLES = '''
 # look at ovirt_auth module to see how to reuse authentication:
 
 # Add host with username/password supporting SR-IOV.
-# Note that override_iptables is false by default in oVirt:
+# Note that override_iptables is false by default in oVirt/RHV:
 - ovirt_hosts:
     cluster: Default
     name: myhost
@@ -153,6 +179,29 @@ EXAMPLES = '''
     state: upgraded
     name: myhost
 
+# discover iscsi targets
+- ovirt_hosts:
+    state: iscsidiscover
+    name: myhost
+    iscsi:
+      username: iscsi_user
+      password: secret
+      address: 10.34.61.145
+      port: 3260
+
+
+# login to iscsi targets
+- ovirt_hosts:
+    state: iscsilogin
+    name: myhost
+    iscsi:
+      username: iscsi_user
+      password: secret
+      address: 10.34.61.145
+      target: "iqn.2015-07.com.mlipchuk2.redhat:444"
+      port: 3260
+
+
 # Reinstall host using public key
 - ovirt_hosts:
     state: reinstalled
@@ -173,11 +222,17 @@ id:
     type: str
     sample: 7de90f31-222c-436c-a1ca-7e655bd5b60c
 host:
-    description: "Dictionary of all the host attributes. Host attributes can be found on your oVirt instance
-                  at following url: https://ovirt.example.com/ovirt-engine/api/model#types/host."
+    description: "Dictionary of all the host attributes. Host attributes can be found on your oVirt/RHV instance
+                  at following url: http://ovirt.github.io/ovirt-engine-api-model/master/#types/host."
     returned: On success if host is found.
+    type: dict
+iscsi_targets:
+    description: "List of host iscsi targets"
+    returned: On success if host is found and state is iscsidiscover.
+    type: list
 '''
 
+import time
 import traceback
 
 try:
@@ -193,6 +248,7 @@ from ansible.module_utils.ovirt import (
     check_sdk,
     create_connection,
     equal,
+    get_id_by_name,
     ovirt_full_argument_spec,
     wait,
 )
@@ -202,38 +258,42 @@ class HostsModule(BaseModule):
 
     def build_entity(self):
         return otypes.Host(
-            name=self._module.params['name'],
+            name=self.param('name'),
             cluster=otypes.Cluster(
-                name=self._module.params['cluster']
-            ) if self._module.params['cluster'] else None,
-            comment=self._module.params['comment'],
-            address=self._module.params['address'],
-            root_password=self._module.params['password'],
+                name=self.param('cluster')
+            ) if self.param('cluster') else None,
+            comment=self.param('comment'),
+            address=self.param('address'),
+            root_password=self.param('password'),
             ssh=otypes.Ssh(
                 authentication_method=otypes.SshAuthenticationMethod.PUBLICKEY,
-            ) if self._module.params['public_key'] else None,
+            ) if self.param('public_key') else None,
             kdump_status=otypes.KdumpStatus(
-                self._module.params['kdump_integration']
-            ) if self._module.params['kdump_integration'] else None,
+                self.param('kdump_integration')
+            ) if self.param('kdump_integration') else None,
             spm=otypes.Spm(
-                priority=self._module.params['spm_priority'],
-            ) if self._module.params['spm_priority'] else None,
-            override_iptables=self._module.params['override_iptables'],
+                priority=self.param('spm_priority'),
+            ) if self.param('spm_priority') else None,
+            override_iptables=self.param('override_iptables'),
             display=otypes.Display(
-                address=self._module.params['override_display'],
-            ) if self._module.params['override_display'] else None,
+                address=self.param('override_display'),
+            ) if self.param('override_display') else None,
             os=otypes.OperatingSystem(
-                custom_kernel_cmdline=' '.join(self._module.params['kernel_params']),
-            ) if self._module.params['kernel_params'] else None,
+                custom_kernel_cmdline=' '.join(self.param('kernel_params')),
+            ) if self.param('kernel_params') else None,
+            power_management=otypes.PowerManagement(
+                enabled=self.param('power_management_enabled'),
+            ) if self.param('power_management_enabled') is not None else None,
         )
 
     def update_check(self, entity):
-        kernel_params = self._module.params.get('kernel_params')
+        kernel_params = self.param('kernel_params')
         return (
-            equal(self._module.params.get('comment'), entity.comment) and
-            equal(self._module.params.get('kdump_integration'), entity.kdump_status) and
-            equal(self._module.params.get('spm_priority'), entity.spm.priority) and
-            equal(self._module.params.get('override_display'), getattr(entity.display, 'address', None)) and
+            equal(self.param('comment'), entity.comment) and
+            equal(self.param('kdump_integration'), entity.kdump_status) and
+            equal(self.param('spm_priority'), entity.spm.priority) and
+            equal(self.param('power_management_enabled'), entity.power_management.enabled) and
+            equal(self.param('override_display'), getattr(entity.display, 'address', None)) and
             equal(
                 sorted(kernel_params) if kernel_params else None,
                 sorted(entity.os.custom_kernel_cmdline.split(' '))
@@ -248,21 +308,37 @@ class HostsModule(BaseModule):
             wait_condition=lambda h: h.status == hoststate.MAINTENANCE,
         )
 
-    def post_update(self, entity):
-        if entity.status != hoststate.UP and self._module.params['state'] == 'present':
-            if not self._module.check_mode:
-                self._service.host_service(entity.id).activate()
-            self.changed = True
-
     def post_reinstall(self, host):
         wait(
             service=self._service.service(host.id),
             condition=lambda h: h.status != hoststate.MAINTENANCE,
             fail_condition=failed_state,
-            wait=self._module.params['wait'],
-            timeout=self._module.params['timeout'],
+            wait=self.param('wait'),
+            timeout=self.param('timeout'),
         )
 
+    def failed_state_after_reinstall(self, host, count=0):
+        if host.status in [
+            hoststate.ERROR,
+            hoststate.INSTALL_FAILED,
+            hoststate.NON_OPERATIONAL,
+        ]:
+            return True
+
+        # If host is in non-responsive state after upgrade/install
+        # let's wait for few seconds and re-check again the state:
+        if host.status == hoststate.NON_RESPONSIVE:
+            if count <= 3:
+                time.sleep(20)
+                return self.failed_state_after_reinstall(
+                    self._service.service(host.id).get(),
+                    count + 1,
+                )
+            else:
+                return True
+
+        return False
+
 
 def failed_state(host):
     return host.status in [
@@ -308,13 +384,15 @@ def control_state(host_module):
             fail_condition=failed_state,
         )
 
+    return host
+
 
 def main():
     argument_spec = ovirt_full_argument_spec(
         state=dict(
             choices=[
                 'present', 'absent', 'maintenance', 'upgraded', 'started',
-                'restarted', 'stopped', 'reinstalled',
+                'restarted', 'stopped', 'reinstalled', 'iscsidiscover', 'iscsilogin'
             ],
             default='present',
         ),
@@ -332,10 +410,18 @@ def main():
         override_display=dict(default=None),
         kernel_params=dict(default=None, type='list'),
         hosted_engine=dict(default=None, choices=['deploy', 'undeploy']),
+        power_management_enabled=dict(default=None, type='bool'),
+        activate=dict(default=True, type='bool'),
+        iscsi=dict(default=None, type='dict'),
+        check_upgrade=dict(default=True, type='bool'),
     )
     module = AnsibleModule(
         argument_spec=argument_spec,
         supports_check_mode=True,
+        required_if=[
+            ['state', 'iscsidiscover', ['iscsi']],
+            ['state', 'iscsilogin', ['iscsi']]
+        ]
     )
     check_sdk(module)
 
@@ -350,19 +436,22 @@ def main():
         )
 
         state = module.params['state']
-        control_state(hosts_module)
+        host = control_state(hosts_module)
         if state == 'present':
-            hosts_module.create(
+            ret = hosts_module.create(
                 deploy_hosted_engine=(
                     module.params.get('hosted_engine') == 'deploy'
                 ) if module.params.get('hosted_engine') is not None else None,
+                result_state=hoststate.UP if host is None else None,
+                fail_condition=hosts_module.failed_state_after_reinstall if host is None else lambda h: False,
             )
-            ret = hosts_module.action(
-                action='activate',
-                action_condition=lambda h: h.status == hoststate.MAINTENANCE,
-                wait_condition=lambda h: h.status == hoststate.UP,
-                fail_condition=failed_state,
-            )
+            if module.params['activate'] and host is not None:
+                ret = hosts_module.action(
+                    action='activate',
+                    action_condition=lambda h: h.status != hoststate.UP,
+                    wait_condition=lambda h: h.status == hoststate.UP,
+                    fail_condition=failed_state,
+                )
         elif state == 'absent':
             ret = hosts_module.remove()
         elif state == 'maintenance':
@@ -374,11 +463,66 @@ def main():
             )
             ret = hosts_module.create()
         elif state == 'upgraded':
+            result_state = hoststate.MAINTENANCE if host.status == hoststate.MAINTENANCE else hoststate.UP
+            events_service = connection.system_service().events_service()
+            last_event = events_service.list(max=1)[0]
+
+            if module.params['check_upgrade']:
+                hosts_module.action(
+                    action='upgrade_check',
+                    action_condition=lambda host: not host.update_available,
+                    wait_condition=lambda host: host.update_available or (
+                        len([
+                            event
+                            for event in events_service.list(
+                                from_=int(last_event.id),
+                                search='type=885 and host.name=%s' % host.name,
+                            )
+                        ]) > 0
+                    ),
+                    fail_condition=lambda host: len([
+                        event
+                        for event in events_service.list(
+                            from_=int(last_event.id),
+                            search='type=839 or type=887 and host.name=%s' % host.name,
+                        )
+                    ]) > 0,
+                )
+                # Set to False, because upgrade_check isn't 'changing' action:
+                hosts_module._changed = False
             ret = hosts_module.action(
                 action='upgrade',
                 action_condition=lambda h: h.update_available,
-                wait_condition=lambda h: h.status == hoststate.UP,
-                fail_condition=failed_state,
+                wait_condition=lambda h: h.status == result_state,
+                post_action=lambda h: time.sleep(module.params['poll_interval']),
+                fail_condition=hosts_module.failed_state_after_reinstall,
+            )
+        elif state == 'iscsidiscover':
+            host_id = get_id_by_name(hosts_service, module.params['name'])
+            iscsi_targets = hosts_service.service(host_id).iscsi_discover(
+                iscsi=otypes.IscsiDetails(
+                    port=int(module.params['iscsi']['port']) if module.params['iscsi']['port'].isdigit() else None,
+                    username=module.params['iscsi']['username'],
+                    password=module.params['iscsi']['password'],
+                    address=module.params['iscsi']['address'],
+                ),
+            )
+            ret = {
+                'changed': False,
+                'id': host_id,
+                'iscsi_targets': iscsi_targets,
+            }
+        elif state == 'iscsilogin':
+            host_id = get_id_by_name(hosts_service, module.params['name'])
+            ret = hosts_module.action(
+                action='iscsi_login',
+                iscsi=otypes.IscsiDetails(
+                    port=int(module.params['iscsi']['port']) if module.params['iscsi']['port'].isdigit() else None,
+                    username=module.params['iscsi']['username'],
+                    password=module.params['iscsi']['password'],
+                    address=module.params['iscsi']['address'],
+                    target=module.params['iscsi']['target'],
+                ),
             )
         elif state == 'started':
             ret = hosts_module.action(
@@ -424,7 +568,7 @@ def main():
                 action_condition=lambda h: h.status == hoststate.MAINTENANCE,
                 post_action=hosts_module.post_reinstall,
                 wait_condition=lambda h: h.status == hoststate.MAINTENANCE,
-                fail_condition=failed_state,
+                fail_condition=hosts_module.failed_state_after_reinstall,
                 host=otypes.Host(
                     override_iptables=module.params['override_iptables'],
                 ) if module.params['override_iptables'] else None,
