@@ -1,82 +1,73 @@
 #!/usr/bin/python
-# This file is part of Ansible
-#
-# Ansible is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# Ansible is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
+# Copyright: (c) 2017, Ansible Project
+# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
-ANSIBLE_METADATA = {'metadata_version': '1.0',
+from __future__ import absolute_import, division, print_function
+__metaclass__ = type
+
+ANSIBLE_METADATA = {'metadata_version': '1.1',
                     'status': ['stableinterface'],
                     'supported_by': 'core'}
 
-
-DOCUMENTATION = '''
+DOCUMENTATION = r'''
 ---
 module: stat
 version_added: "1.3"
-short_description: retrieve file or file system status
+short_description: Retrieve file or file system status
 description:
      - Retrieves facts for a file similar to the linux/unix 'stat' command.
+     - For Windows targets, use the M(win_stat) module instead.
 options:
   path:
     description:
-      - The full path of the file/object to get the facts of
+      - The full path of the file/object to get the facts of.
     required: true
-    default: null
   follow:
     description:
-      - Whether to follow symlinks
-    required: false
-    default: no
+      - Whether to follow symlinks.
+    choices: [ 'no', 'yes' ]
+    default: 'no'
   get_md5:
     description:
-      - Whether to return the md5 sum of the file.  Will return None if we're
-        unable to use md5 (Common for FIPS-140 compliant systems)
-    required: false
-    default: yes
+      - Whether to return the md5 sum of the file.
+      - Will return None if not a regular file or if we're
+        unable to use md5 (Common for FIPS-140 compliant systems).
+    choices: [ 'no', 'yes' ]
+    default: 'yes'
   get_checksum:
     description:
-      - Whether to return a checksum of the file (default sha1)
-    required: false
-    default: yes
+      - Whether to return a checksum of the file (default sha1).
+    choices: [ 'no', 'yes' ]
+    default: 'yes'
     version_added: "1.8"
   checksum_algorithm:
     description:
       - Algorithm to determine checksum of file. Will throw an error if the
         host is unable to use specified algorithm.
-    required: false
-    choices: [ 'sha1', 'sha224', 'sha256', 'sha384', 'sha512' ]
+    choices: [ sha1, sha224, sha256, sha384, sha512 ]
     default: sha1
-    aliases: [ 'checksum_algo', 'checksum' ]
+    aliases: [ checksum, checksum_algo ]
     version_added: "2.0"
   get_mime:
     description:
       - Use file magic and return data about the nature of the file. this uses
         the 'file' utility found on most Linux/Unix systems.
       - This will add both `mime_type` and 'charset' fields to the return, if possible.
-      - In 2.3 this option changed from 'mime' to 'get_mime' and the default changed to 'Yes'
-    required: false
-    choices: [ Yes, No ]
-    default: Yes
+      - In 2.3 this option changed from 'mime' to 'get_mime' and the default changed to 'Yes'.
+    choices: [ 'no', 'yes' ]
+    default: 'yes'
     version_added: "2.1"
-    aliases: [ 'mime', 'mime_type', 'mime-type' ]
+    aliases: [ mime, mime_type, mime-type ]
   get_attributes:
     description:
       - Get file attributes using lsattr tool if present.
-    required: false
-    default: True
+    choices: [ 'no', 'yes' ]
+    default: 'yes'
     version_added: "2.3"
-    aliases: [ 'attributes', 'attr' ]
-author: "Bruce Pennypacker (@bpennypacker)"
+    aliases: [ attr, attributes ]
+notes:
+     - For Windows targets, use the M(win_stat) module instead.
+author: Bruce Pennypacker (@bpennypacker)
 '''
 
 EXAMPLES = '''
@@ -135,11 +126,11 @@ EXAMPLES = '''
     checksum_algorithm: sha256
 '''
 
-RETURN = '''
+RETURN = r'''
 stat:
     description: dictionary containing all the stat data, some platforms might add additional fields
     returned: success
-    type: dictionary
+    type: complex
     contains:
         exists:
             description: if the destination path actually exists or not
@@ -292,10 +283,16 @@ stat:
             type: boolean
             sample: False
         lnk_source:
-            description: Original path
+            description: Target of the symlink normalized for the remote filesystem
             returned: success, path exists and user can read stats and the path is a symbolic link
             type: string
             sample: /home/foobar/21102015-1445431274-908472971
+        lnk_target:
+            description: Target of the symlink.  Note that relative paths remain relative
+            returned: success, path exists and user can read stats and the path is a symbolic link
+            type: string
+            sample: ../foobar/21102015-1445431274-908472971
+            version_added: 2.4
         md5:
             description: md5 hash of the path
             returned: success, path exists and user can read stats and path
@@ -353,7 +350,7 @@ stat:
         attributes:
             description: list of file attributes
             returned: success, path exists and user can execute the path
-            type: boolean
+            type: list
             sample: [ immutable, extent ]
             version_added: 2.3
 '''
@@ -365,10 +362,10 @@ import pwd
 import stat
 
 # import module snippets
-from ansible.module_utils.basic import AnsibleModule, format_attributes
-from ansible.module_utils.pycompat24 import get_exception
+from ansible.module_utils.basic import AnsibleModule
 from ansible.module_utils._text import to_bytes
 
+
 def format_output(module, path, st):
     mode = st.st_mode
 
@@ -406,12 +403,12 @@ def format_output(module, path, st):
         isgid=bool(mode & stat.S_ISGID),
     )
 
-    # Platform dependant flags:
+    # Platform dependent flags:
     for other in [
             # Some Linux
-            ('st_blocks','blocks'),
+            ('st_blocks', 'blocks'),
             ('st_blksize', 'block_size'),
-            ('st_rdev','device_type'),
+            ('st_rdev', 'device_type'),
             ('st_flags', 'flags'),
             # Some Berkley based
             ('st_gen', 'generation'),
@@ -428,7 +425,6 @@ def format_output(module, path, st):
         if hasattr(st, other[0]):
             output[other[1]] = getattr(st, other[0])
 
-
     return output
 
 
@@ -436,16 +432,16 @@ def main():
     module = AnsibleModule(
         argument_spec=dict(
             path=dict(required=True, type='path'),
-            follow=dict(default='no', type='bool'),
-            get_md5=dict(default='yes', type='bool'),
-            get_checksum=dict(default='yes', type='bool'),
-            get_mime=dict(default=True, type='bool', aliases=['mime', 'mime_type', 'mime-type']),
-            get_attributes=dict(default=True, type='bool', aliases=['attributes', 'attr']),
-            checksum_algorithm=dict(default='sha1', type='str',
+            follow=dict(type='bool', default='no'),
+            get_md5=dict(type='bool', default='yes'),
+            get_checksum=dict(type='bool', default='yes'),
+            get_mime=dict(type='bool', default='yes', aliases=['mime', 'mime_type', 'mime-type']),
+            get_attributes=dict(type='bool', default='yes', aliases=['attr', 'attributes']),
+            checksum_algorithm=dict(type='str', default='sha1',
                                     choices=['sha1', 'sha224', 'sha256', 'sha384', 'sha512'],
-                                    aliases=['checksum_algo', 'checksum']),
+                                    aliases=['checksum', 'checksum_algo']),
         ),
-        supports_check_mode=True
+        supports_check_mode=True,
     )
 
     path = module.params.get('path')
@@ -463,8 +459,7 @@ def main():
             st = os.stat(b_path)
         else:
             st = os.lstat(b_path)
-    except OSError:
-        e = get_exception()
+    except OSError as e:
         if e.errno == errno.ENOENT:
             output = {'exists': False}
             module.exit_json(changed=False, stat=output)
@@ -476,19 +471,20 @@ def main():
 
     # resolved permissions
     for perm in [('readable', os.R_OK), ('writeable', os.W_OK), ('executable', os.X_OK)]:
-        output[perm[0]] = os.access(path, perm[1])
+        output[perm[0]] = os.access(b_path, perm[1])
 
     # symlink info
     if output.get('islnk'):
-        output['lnk_source'] = os.path.realpath(path)
+        output['lnk_source'] = os.path.realpath(b_path)
+        output['lnk_target'] = os.readlink(b_path)
 
-    try: # user data
+    try:  # user data
         pw = pwd.getpwuid(st.st_uid)
         output['pw_name'] = pw.pw_name
     except:
         pass
 
-    try: # group data
+    try:  # group data
         grp_info = grp.getgrgid(st.st_gid)
         output['gr_name'] = grp_info.gr_name
     except:
@@ -499,19 +495,19 @@ def main():
         if get_md5:
             # Will fail on FIPS-140 compliant systems
             try:
-                output['md5'] = module.md5(path)
+                output['md5'] = module.md5(b_path)
             except ValueError:
                 output['md5'] = None
 
         if get_checksum:
-            output['checksum'] = module.digest_from_file(path, checksum_algorithm)
+            output['checksum'] = module.digest_from_file(b_path, checksum_algorithm)
 
     # try to get mime data if requested
     if get_mime:
         output['mimetype'] = output['charset'] = 'unknown'
         mimecmd = module.get_bin_path('file')
         if mimecmd:
-            mimecmd = [mimecmd, '-i', path]
+            mimecmd = [mimecmd, '-i', b_path]
             try:
                 rc, out, err = module.run_command(mimecmd)
                 if rc == 0:
@@ -526,12 +522,13 @@ def main():
         output['version'] = None
         output['attributes'] = []
         output['attr_flags'] = ''
-        out = module.get_file_attributes(path)
+        out = module.get_file_attributes(b_path)
         for x in ('version', 'attributes', 'attr_flags'):
             if x in out:
                 output[x] = out[x]
 
     module.exit_json(changed=False, stat=output)
 
+
 if __name__ == '__main__':
     main()
