@@ -2,23 +2,13 @@
 # -*- coding: utf-8 -*-
 
 # (c) 2015, Benjamin Copeland (@bhcopeland) <ben@copeland.me.uk>
-#
-# This file is part of Ansible
-#
-# Ansible is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# Ansible is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Ansible. If not, see <http://www.gnu.org/licenses/>.
+# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
-ANSIBLE_METADATA = {'metadata_version': '1.0',
+from __future__ import absolute_import, division, print_function
+__metaclass__ = type
+
+
+ANSIBLE_METADATA = {'metadata_version': '1.1',
                     'status': ['preview'],
                     'supported_by': 'community'}
 
@@ -190,6 +180,11 @@ EXAMPLES = '''
 RETURN = ''' # '''
 
 import datetime
+import json
+
+from ansible.module_utils.basic import AnsibleModule
+from ansible.module_utils._text import to_native
+from ansible.module_utils.urls import open_url
 
 
 def get_api_auth_headers(api_id, api_key, url, statuspage):
@@ -210,8 +205,8 @@ def get_api_auth_headers(api_id, api_key, url, statuspage):
         else:
             auth_headers = headers
             auth_content = data
-    except:
-        return 1, None, None, e
+    except Exception as e:
+        return 1, None, None, to_native(e)
     return 0, auth_headers, auth_content, None
 
 
@@ -320,9 +315,8 @@ def create_maintenance(auth_headers, url, statuspage, host_ids,
 
         if data["status"]["error"] == "yes":
             return 1, None, data["status"]["message"]
-    except Exception:
-        e = get_exception()
-        return 1, None, str(e)
+    except Exception as e:
+        return 1, None, to_native(e)
     return 0, None, None
 
 
@@ -339,9 +333,8 @@ def delete_maintenance(auth_headers, url, statuspage, maintenance_id):
         data = json.loads(response.read())
         if data["status"]["error"] == "yes":
             return 1, None, "Invalid maintenance_id"
-    except Exception:
-        e = get_exception()
-        return 1, None, str(e)
+    except Exception as e:
+        return 1, None, to_native(e)
     return 0, None, None
 
 
@@ -475,7 +468,6 @@ def main():
                 module.fail_json(
                     msg="Failed to delete maintenance: %s" % error)
 
-from ansible.module_utils.basic import *
-from ansible.module_utils.urls import *
+
 if __name__ == '__main__':
     main()
