@@ -16,9 +16,9 @@
 # along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-ANSIBLE_METADATA = {'metadata_version': '1.0',
+ANSIBLE_METADATA = {'metadata_version': '1.1',
                     'status': ['preview'],
-                    'supported_by': 'community'}
+                    'supported_by': 'network'}
 
 
 DOCUMENTATION = '''
@@ -31,63 +31,64 @@ description:
     - Manages physical attributes of interfaces of NX-OS switches.
 author: Jason Edelman (@jedelman8)
 notes:
-    - This module is also used to create logical interfaces such as
-      svis and loopbacks.
-    - Be cautious of platform specific idiosyncrasies. For example,
-      when you default a loopback interface, the admin state toggles
-      on certain versions of NX-OS.
-    - The M(nxos_overlay_global) C(anycast_gateway_mac) attribute must be
-      set before setting the C(fabric_forwarding_anycast_gateway) property.
+  - Tested against NXOSv 7.3.(0)D1(1) on VIRL
+  - This module is also used to create logical interfaces such as
+    svis and loopbacks.
+  - Be cautious of platform specific idiosyncrasies. For example,
+    when you default a loopback interface, the admin state toggles
+    on certain versions of NX-OS.
+  - The M(nxos_overlay_global) C(anycast_gateway_mac) attribute must be
+    set before setting the C(fabric_forwarding_anycast_gateway) property.
 options:
-    interface:
-        description:
-            - Full name of interface, i.e. Ethernet1/1, port-channel10.
-        required: true
-        default: null
-    interface_type:
-        description:
-            - Interface type to be unconfigured from the device.
-        required: false
-        default: null
-        choices: ['loopback', 'portchannel', 'svi', 'nve']
-        version_added: "2.2"
-    admin_state:
-        description:
-            - Administrative state of the interface.
-        required: false
-        default: up
-        choices: ['up','down']
+  interface:
     description:
-        description:
-            - Interface description.
-        required: false
-        default: null
-    mode:
-        description:
-            - Manage Layer 2 or Layer 3 state of the interface.
-        required: false
-        default: null
-        choices: ['layer2','layer3']
-    ip_forward:
-        description:
-            - Enable/Disable ip forward feature on SVIs.
-        required: false
-        default: null
-        choices: ['enable','disable']
-        version_added: "2.2"
-    fabric_forwarding_anycast_gateway:
-        description:
-            - Associate SVI with anycast gateway under VLAN configuration mode.
-        required: false
-        default: null
-        choices: ['true','false']
-        version_added: "2.2"
-    state:
-        description:
-            - Specify desired state of the resource.
-        required: true
-        default: present
-        choices: ['present','absent','default']
+      - Full name of interface, i.e. Ethernet1/1, port-channel10.
+    required: true
+    default: null
+  interface_type:
+    description:
+      - Interface type to be unconfigured from the device.
+    required: false
+    default: null
+    choices: ['loopback', 'portchannel', 'svi', 'nve']
+    version_added: "2.2"
+  admin_state:
+    description:
+      - Administrative state of the interface.
+    required: false
+    default: up
+    choices: ['up','down']
+  description:
+    description:
+      - Interface description.
+    required: false
+    default: null
+  mode:
+    description:
+      - Manage Layer 2 or Layer 3 state of the interface.
+    required: false
+    default: null
+    choices: ['layer2','layer3']
+  ip_forward:
+    description:
+      - Enable/Disable ip forward feature on SVIs.
+    required: false
+    default: null
+    choices: ['enable','disable']
+    version_added: "2.2"
+  fabric_forwarding_anycast_gateway:
+    description:
+      - Associate SVI with anycast gateway under VLAN configuration mode.
+    required: false
+    default: null
+    choices: ['true','false']
+    version_added: "2.2"
+  state:
+    description:
+      - Specify desired state of the resource.
+    required: true
+    default: present
+    choices: ['present','absent','default']
 '''
 
 EXAMPLES = '''
@@ -121,58 +122,30 @@ EXAMPLES = '''
     - svi
     - nve
 
-- name: Admin up all ethernet interfaces
+- name: Admin up all loopback interfaces
   nxos_interface:
-    interface: ethernet
-    host: 68.170.147.165
+    interface: loopback 0-1023
     admin_state: up
 
-- name: Admin down ALL interfaces (physical and logical)
+- name: Admin down all loopback interfaces
   nxos_interface:
-    interface: all
-    host: 68.170.147.165
+    interface: looback 0-1023
     admin_state: down
 '''
 
 RETURN = '''
-proposed:
-    description: k/v pairs of parameters passed into module
-    returned: always
-    type: dict
-    sample: {"admin_state": "down"}
-existing:
-    description: k/v pairs of existing switchport
-    returned: always
-    type: dict
-    sample:  {"admin_state": "up", "description": "None",
-              "interface": "port-channel101", "mode": "layer2",
-              "type": "portchannel", "ip_forward": "enable"}
-end_state:
-    description: k/v pairs of switchport after module execution
-    returned: always
-    type: dict
-    sample:  {"admin_state": "down", "description": "None",
-              "interface": "port-channel101", "mode": "layer2",
-              "type": "portchannel", "ip_forward": "enable"}
-updates:
+commands:
     description: command list sent to the device
     returned: always
     type: list
     sample: ["interface port-channel101", "shutdown"]
-changed:
-    description: check to see if a change was made on the device
-    returned: always
-    type: boolean
-    sample: true
 '''
 
-from ansible.module_utils.nxos import get_config, load_config, run_commands
+from ansible.module_utils.nxos import load_config, run_commands
 from ansible.module_utils.nxos import nxos_argument_spec, check_args
 from ansible.module_utils.basic import AnsibleModule
 
 
-
-
 def is_default_interface(interface, module):
     """Checks to see if interface exists and if it is a default config
     Args:
@@ -183,11 +156,10 @@ def is_default_interface(interface, module):
         False: if it does not have a default config
         DNE (str): if the interface does not exist - loopbacks, SVIs, etc.
     """
-    command = 'show run interface ' + interface
+    command = 'show run interface {0}'.format(interface)
 
     try:
-        body = execute_show_command(command, module,
-                                    command_type='cli_show_ascii')[0]
+        body = execute_show_command(command, module)[0]
     except IndexError:
         body = ''
 
@@ -242,23 +214,23 @@ def get_manual_interface_attributes(interface, module):
     """
 
     if get_interface_type(interface) == 'svi':
-        command = 'show interface ' + interface
+        command = 'show run interface {0} all'.format(interface)
         try:
-            body = execute_modified_show_for_cli_text(command, module)[0]
-        except (IndexError, ShellError):
+            # body = run_commands(module, [command])[0]
+            body = execute_show_command(command, module)[0]
+        except IndexError:
             return None
+        if body:
+            command_list = body.split('\n')
+            desc = None
+            admin_state = 'down'
+            for each in command_list:
+                if 'description' in each:
+                    desc = each.lstrip().split("description")[1].lstrip()
+                elif 'no shutdown' in each:
+                    admin_state = 'up'
 
-        command_list = body.split('\n')
-        desc = None
-        admin_state = 'up'
-        for each in command_list:
-            if 'Description:' in each:
-                line = each.split('Description:')
-                desc = line[1].strip().split('MTU')[0].strip()
-            elif 'Administratively down' in each:
-                admin_state = 'down'
-
-        return dict(description=desc, admin_state=admin_state)
+            return dict(description=desc, admin_state=admin_state)
     else:
         return None
 
@@ -299,78 +271,82 @@ def get_interface(intf, module):
     key_map = {}
     interface = {}
 
-    command = 'show interface ' + intf
+    command = 'show interface {0}'.format(intf)
     try:
         body = execute_show_command(command, module)[0]
     except IndexError:
-        body = []
-
+        return {}
     if body:
-        interface_table = body['TABLE_interface']['ROW_interface']
-        intf_type = get_interface_type(intf)
-        if intf_type in ['portchannel', 'ethernet']:
-            if not interface_table.get('eth_mode'):
-                interface_table['eth_mode'] = 'layer3'
+        try:
+            interface_table = body['TABLE_interface']['ROW_interface']
+        except KeyError:
+            return {}
+        if interface_table:
+            if interface_table.get('eth_mode') == 'fex-fabric':
+                module.fail_json(msg='nxos_interface does not support interfaces with mode "fex-fabric"')
+            intf_type = get_interface_type(intf)
+            if intf_type in ['portchannel', 'ethernet']:
+                if not interface_table.get('eth_mode'):
+                    interface_table['eth_mode'] = 'layer3'
 
-        if intf_type == 'ethernet':
-            key_map.update(base_key_map)
-            key_map.update(mode_map)
-            temp_dict = apply_key_map(key_map, interface_table)
-            temp_dict = apply_value_map(mode_value_map, temp_dict)
-            interface.update(temp_dict)
+            if intf_type == 'ethernet':
+                key_map.update(base_key_map)
+                key_map.update(mode_map)
+                temp_dict = apply_key_map(key_map, interface_table)
+                temp_dict = apply_value_map(mode_value_map, temp_dict)
+                interface.update(temp_dict)
 
-        elif intf_type == 'svi':
-            key_map.update(svi_map)
-            temp_dict = apply_key_map(key_map, interface_table)
-            interface.update(temp_dict)
-            attributes = get_manual_interface_attributes(intf, module)
-            interface['admin_state'] = str(attributes.get('admin_state',
-                                                          'nxapibug'))
-            interface['description'] = str(attributes.get('description',
-                                                          'nxapi_bug'))
-            command = 'show run interface ' + intf
-            body = execute_show_command(command, module,
-                                        command_type='cli_show_ascii')[0]
-            if 'ip forward' in body:
-                interface['ip_forward'] = 'enable'
-            else:
-                interface['ip_forward'] = 'disable'
-            if 'fabric forwarding mode anycast-gateway' in body:
-                interface['fabric_forwarding_anycast_gateway'] = True
-            else:
-                interface['fabric_forwarding_anycast_gateway'] = False
+            elif intf_type == 'svi':
+                key_map.update(svi_map)
+                temp_dict = apply_key_map(key_map, interface_table)
+                interface.update(temp_dict)
+                attributes = get_manual_interface_attributes(intf, module)
+                interface['admin_state'] = str(attributes.get('admin_state',
+                                                              'nxapibug'))
+                interface['description'] = str(attributes.get('description',
+                                                              'nxapi_bug'))
+                command = 'show run interface {0}'.format(intf)
+                body = execute_show_command(command, module)[0]
+                if 'ip forward' in body:
+                    interface['ip_forward'] = 'enable'
+                else:
+                    interface['ip_forward'] = 'disable'
+                if 'fabric forwarding mode anycast-gateway' in body:
+                    interface['fabric_forwarding_anycast_gateway'] = True
+                else:
+                    interface['fabric_forwarding_anycast_gateway'] = False
 
-        elif intf_type == 'loopback':
-            key_map.update(base_key_map)
-            key_map.pop('admin_state')
-            key_map.update(loop_map)
-            temp_dict = apply_key_map(key_map, interface_table)
-            if not temp_dict.get('description'):
-                temp_dict['description'] = "None"
-            interface.update(temp_dict)
+            elif intf_type == 'loopback':
+                key_map.update(base_key_map)
+                key_map.pop('admin_state')
+                key_map.update(loop_map)
+                temp_dict = apply_key_map(key_map, interface_table)
+                if not temp_dict.get('description'):
+                    temp_dict['description'] = "None"
+                interface.update(temp_dict)
 
-        elif intf_type == 'management':
-            key_map.update(base_key_map)
-            temp_dict = apply_key_map(key_map, interface_table)
-            interface.update(temp_dict)
+            elif intf_type == 'management':
+                key_map.update(base_key_map)
+                temp_dict = apply_key_map(key_map, interface_table)
+                interface.update(temp_dict)
 
-        elif intf_type == 'portchannel':
-            key_map.update(base_key_map)
-            key_map.update(mode_map)
-            temp_dict = apply_key_map(key_map, interface_table)
-            temp_dict = apply_value_map(mode_value_map, temp_dict)
-            if not temp_dict.get('description'):
-                temp_dict['description'] = "None"
-            interface.update(temp_dict)
+            elif intf_type == 'portchannel':
+                key_map.update(base_key_map)
+                key_map.update(mode_map)
+                temp_dict = apply_key_map(key_map, interface_table)
+                temp_dict = apply_value_map(mode_value_map, temp_dict)
+                if not temp_dict.get('description'):
+                    temp_dict['description'] = "None"
+                interface.update(temp_dict)
 
-        elif intf_type == 'nve':
-            key_map.update(base_key_map)
-            temp_dict = apply_key_map(key_map, interface_table)
-            if not temp_dict.get('description'):
-                temp_dict['description'] = "None"
-            interface.update(temp_dict)
+            elif intf_type == 'nve':
+                key_map.update(base_key_map)
+                temp_dict = apply_key_map(key_map, interface_table)
+                if not temp_dict.get('description'):
+                    temp_dict['description'] = "None"
+                interface.update(temp_dict)
 
-    interface['type'] = intf_type
+        interface['type'] = intf_type
 
     return interface
 
@@ -395,11 +371,11 @@ def get_interfaces_dict(module):
             keys.  Each value is a list of interfaces of given interface (key)
             type.
     """
-    command = 'show interface status'
+    command = 'show interface'
     try:
         body = execute_show_command(command, module)[0]
     except IndexError:
-        body = {}
+        return {}
 
     interfaces = {
         'ethernet': [],
@@ -409,14 +385,15 @@ def get_interfaces_dict(module):
         'portchannel': [],
         'nve': [],
         'unknown': []
-        }
+    }
 
-    interface_list = body.get('TABLE_interface')['ROW_interface']
-    for index in interface_list:
-        intf = index ['interface']
-        intf_type = get_interface_type(intf)
+    if body:
+        interface_list = body['TABLE_interface']['ROW_interface']
+        for index in interface_list:
+            intf = index['interface']
+            intf_type = get_interface_type(intf)
 
-        interfaces[intf_type].append(intf)
+            interfaces[intf_type].append(intf)
 
     return interfaces
 
@@ -521,12 +498,13 @@ def get_interface_config_commands(interface, intf, existing):
             commands.append('no fabric forwarding mode anycast-gateway')
 
     if commands:
-        commands.insert(0, 'interface ' + intf)
+        commands.insert(0, 'interface {0}'.format(intf))
 
     return commands
 
 
 def get_admin_state(interface, intf, admin_state):
+    command = ''
     if admin_state == 'up':
         command = 'no shutdown'
     elif admin_state == 'down':
@@ -568,24 +546,17 @@ def smart_existing(module, intf_type, normalized_interface):
     return existing, is_default
 
 
-def execute_show_command(command, module, command_type='cli_show'):
-    if module.params['transport'] == 'cli':
-        command += ' | json'
-        cmds = [command]
-        body = run_commands(module, cmds)
-    elif module.params['transport'] == 'nxapi':
-        cmds = [{'command': command, 'output': 'json'}]
-        body = run_commands(module, cmds)
-    return body
-
-
-def execute_modified_show_for_cli_text(command, module):
-    cmds = [command]
-    if module.params['transport'] == 'cli':
-        body = run_commands(module, cmds)
+def execute_show_command(command, module):
+    if 'show run' not in command:
+        output = 'json'
     else:
-        body = run_commands(module, cmds)
-    body = response
+        output = 'text'
+    cmds = [{
+        'command': command,
+        'output': output,
+    }]
+
+    body = run_commands(module, cmds)
     return body
 
 
@@ -616,26 +587,23 @@ def main():
         admin_state=dict(default='up', choices=['up', 'down'], required=False),
         description=dict(required=False, default=None),
         mode=dict(choices=['layer2', 'layer3'], required=False),
-        interface_type=dict(required=False,
-                            choices=['loopback', 'portchannel', 'svi', 'nve']),
+        interface_type=dict(required=False, choices=['loopback', 'portchannel', 'svi', 'nve']),
         ip_forward=dict(required=False, choices=['enable', 'disable']),
         fabric_forwarding_anycast_gateway=dict(required=False, type='bool'),
-        state=dict(choices=['absent', 'present', 'default'],
-                   default='present', required=False),
-        include_defaults=dict(default=True),
-        config=dict(),
-        save=dict(type='bool', default=False)
+        state=dict(choices=['absent', 'present', 'default'], default='present', required=False)
     )
 
     argument_spec.update(nxos_argument_spec)
 
     module = AnsibleModule(argument_spec=argument_spec,
-                                mutually_exclusive=[['interface', 'interface_type']],
-                                supports_check_mode=True)
+                           mutually_exclusive=[['interface', 'interface_type']],
+                           supports_check_mode=True)
 
     warnings = list()
     check_args(module, warnings)
 
+    results = dict(changed=False, warnings=warnings)
+
     interface = module.params['interface']
     interface_type = module.params['interface_type']
     admin_state = module.params['admin_state']
@@ -658,8 +626,7 @@ def main():
                 module.fail_json(msg='description and mode params are not '
                                      'supported in this module. Use '
                                      'nxos_vxlan_vtep instead.')
-        if ((ip_forward or fabric_forwarding_anycast_gateway) and
-             intf_type != 'svi'):
+        if (ip_forward or fabric_forwarding_anycast_gateway) and intf_type != 'svi':
             module.fail_json(msg='The ip_forward and '
                                  'fabric_forwarding_anycast_gateway features '
                                  ' are only available for SVIs.')
@@ -668,9 +635,8 @@ def main():
                     fabric_forwarding_anycast_gateway=fabric_forwarding_anycast_gateway)
 
         if intf_type == 'unknown':
-            module.fail_json(
-                msg='unknown interface type found-1',
-                interface=interface)
+            module.fail_json(msg='unknown interface type found-1',
+                             interface=interface)
 
         existing, is_default = smart_existing(module, intf_type, normalized_interface)
         proposed = get_proposed(existing, normalized_interface, args)
@@ -678,7 +644,6 @@ def main():
         intf_type = normalized_interface = interface_type
         proposed = dict(interface_type=interface_type)
 
-    changed = False
     commands = []
     if interface:
         delta = dict()
@@ -694,29 +659,22 @@ def main():
                     commands.append(cmds)
         elif state == 'present':
             if not existing:
-                cmds = get_interface_config_commands(proposed,
-                                                     normalized_interface,
-                                                     existing)
+                cmds = get_interface_config_commands(proposed, normalized_interface, existing)
                 commands.append(cmds)
             else:
-                delta = dict(set(proposed.items()).difference(
-                    existing.items()))
+                delta = dict(set(proposed.items()).difference(existing.items()))
                 if delta:
-                    cmds = get_interface_config_commands(delta,
-                                                         normalized_interface,
-                                                         existing)
+                    cmds = get_interface_config_commands(delta, normalized_interface, existing)
                     commands.append(cmds)
         elif state == 'default':
             if is_default is False:
                 cmds = ['default interface {0}'.format(normalized_interface)]
                 commands.append(cmds)
             elif is_default == 'DNE':
-                module.exit_json(msg='interface you are trying to default does'
-                                     ' not exist')
+                module.exit_json(msg='interface you are trying to default does not exist')
     elif interface_type:
         if state == 'present':
-            module.fail_json(msg='The interface_type param can be used '
-                                 'only with state absent.')
+            module.fail_json(msg='The interface_type param can be used only with state absent.')
 
         existing = get_interfaces_dict(module)[interface_type]
         cmds = get_interface_type_removed_cmds(existing)
@@ -730,9 +688,9 @@ def main():
             module.exit_json(changed=True, commands=cmds)
         else:
             load_config(module, cmds)
-            changed = True
+            results['changed'] = True
             if module.params['interface']:
-                if delta.get('mode'): # or delta.get('admin_state'):
+                if delta.get('mode'):
                     # if the mode changes from L2 to L3, the admin state
                     # seems to change after the API call, so adding a second API
                     # call to ensure it's in the desired state.
@@ -744,21 +702,12 @@ def main():
                     cmds.extend(cmds2)
                 end_state, is_default = smart_existing(module, intf_type,
                                                        normalized_interface)
-            else:
-                end_state = get_interfaces_dict(module)[interface_type]
             cmds = [cmd for cmd in cmds if cmd != 'configure']
 
-    results = {}
-    results['proposed'] = proposed
-    results['existing'] = existing
-    results['end_state'] = end_state
-    results['updates'] = cmds
-    results['changed'] = changed
-    results['warnings'] = warnings
+    results['commands'] = cmds
 
     module.exit_json(**results)
 
 
 if __name__ == '__main__':
     main()
-
