@@ -16,10 +16,10 @@ DOCUMENTATION = '''
 ---
 module: panos_security_rule
 short_description: Create security rule policy on PAN-OS devices or Panorama management console.
-description: >
-    - Security policies allow you to enforce rules and take action, and can be as general or specific as needed. The
-    policy rules are compared against the incoming traffic in sequence, and because the first rule that matches the
-    traffic is applied, the more specific rules must precede the more general ones.
+description:
+    - Security policies allow you to enforce rules and take action, and can be as general or specific as needed.
+      The policy rules are compared against the incoming traffic in sequence, and because the first rule that matches the traffic is applied,
+      the more specific rules must precede the more general ones.
 author: "Ivan Bojer (@ivanbojer), Robert Hagen (@rnh556)"
 version_added: "2.4"
 requirements:
