@@ -2,23 +2,13 @@
 # -*- coding: utf-8 -*-
 
 # Copyright 2012 Dag Wieers <dag@wieers.com>
-#
-# This file is part of Ansible
-#
-# Ansible is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# Ansible is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
+# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
-ANSIBLE_METADATA = {'metadata_version': '1.0',
+from __future__ import absolute_import, division, print_function
+__metaclass__ = type
+
+
+ANSIBLE_METADATA = {'metadata_version': '1.1',
                     'status': ['stableinterface'],
                     'supported_by': 'core'}
 
@@ -30,6 +20,7 @@ short_description: Fail with custom message
 description:
      - This module fails the progress with a custom message. It can be
        useful for bailing out when a certain condition is met using C(when).
+     - This module is also supported for Windows targets.
 version_added: "0.8"
 options:
   msg:
@@ -38,6 +29,8 @@ options:
         fail will simply bail out with a generic message.
     required: false
     default: "'Failed as requested from task'"
+notes:
+    - This module is also supported for Windows targets.
 
 author: "Dag Wieers (@dagwieers)"
 '''
