@@ -19,14 +19,14 @@
 from __future__ import (absolute_import, division, print_function)
 __metaclass__ = type
 
-from ansible.compat.six import iteritems, string_types
-
-from ansible.errors import AnsibleParserError,AnsibleError
+from ansible.errors import AnsibleParserError, AnsibleError
+from ansible.module_utils.six import iteritems, string_types
 from ansible.module_utils._text import to_text
-from ansible.plugins import module_loader
 from ansible.parsing.splitter import parse_kv, split_args
+from ansible.plugins.loader import module_loader, action_loader
 from ansible.template import Templar
 
+
 # For filtering out modules correctly below
 RAW_PARAM_MODULES = ([
     'command',
@@ -36,6 +36,10 @@ RAW_PARAM_MODULES = ([
     'script',
     'include',
     'include_vars',
+    'include_tasks',
+    'include_role',
+    'import_tasks',
+    'import_role',
     'add_host',
     'group_by',
     'set_fact',
@@ -43,6 +47,7 @@ RAW_PARAM_MODULES = ([
     'meta',
 ])
 
+
 class ModuleArgsParser:
 
     """
@@ -90,11 +95,11 @@ class ModuleArgsParser:
     Args may also be munged for certain shell command parameters.
     """
 
+    # FIXME: mutable default arg
     def __init__(self, task_ds=dict()):
-        assert isinstance(task_ds, dict)
+        assert isinstance(task_ds, dict), "the type of 'task_ds' should be a dict, but is a %s" % type(task_ds)
         self._task_ds = task_ds
 
-
     def _split_module_string(self, module_string):
         '''
         when module names are expressed like:
@@ -109,7 +114,6 @@ class ModuleArgsParser:
         else:
             return (tokens[0], "")
 
-
     def _handle_shell_weirdness(self, action, args):
         '''
         given an action name and an args dictionary, return the
@@ -140,7 +144,8 @@ class ModuleArgsParser:
                 if templar._contains_vars(additional_args):
                     final_args['_variable_params'] = additional_args
                 else:
-                    raise AnsibleParserError("Complex args containing variables cannot use bare variables, and must use the full variable style ('{{var_name}}')")
+                    raise AnsibleParserError("Complex args containing variables cannot use bare variables, and must use the full variable style "
+                                             "('{{var_name}}')")
             elif isinstance(additional_args, dict):
                 final_args.update(additional_args)
             else:
@@ -163,7 +168,7 @@ class ModuleArgsParser:
                 args.update(tmp_args)
 
         # only internal variables can start with an underscore, so
-        # we don't allow users to set them directy in arguments
+        # we don't allow users to set them directly in arguments
         if args and action not in ('command', 'win_command', 'shell', 'win_shell', 'script', 'raw'):
             for arg in args:
                 arg = to_text(arg)
@@ -250,12 +255,11 @@ class ModuleArgsParser:
         task, dealing with all sorts of levels of fuzziness.
         '''
 
-        thing      = None
+        thing = None
 
-        action      = None
+        action = None
         delegate_to = self._task_ds.get('delegate_to', None)
-        args        = dict()
-
+        args = dict()
 
         # This is the standard YAML form for command-type modules. We grab
         # the args and pass them in as additional arguments, which can/will
@@ -269,7 +273,6 @@ class ModuleArgsParser:
             thing = self._task_ds['action']
             action, args = self._normalize_parameters(thing, action=action, additional_args=additional_args)
 
-
         # local_action
         if 'local_action' in self._task_ds:
             # local_action is similar but also implies a delegate_to
@@ -283,7 +286,7 @@ class ModuleArgsParser:
 
         # walk the input dictionary to see we recognize a module name
         for (item, value) in iteritems(self._task_ds):
-            if item in module_loader or item in ['meta', 'include', 'include_role']:
+            if item in module_loader or item in action_loader or item in ['meta', 'include', 'include_tasks', 'include_role', 'import_tasks', 'import_role']:
                 # finding more than one module name is a problem
                 if action is not None:
                     raise AnsibleParserError("conflicting action statements: %s, %s" % (action, item), obj=self._task_ds)
@@ -291,24 +294,26 @@ class ModuleArgsParser:
                 thing = value
                 action, args = self._normalize_parameters(thing, action=action, additional_args=additional_args)
 
-
         # if we didn't see any module in the task at all, it's not a task really
         if action is None:
             if 'ping' not in module_loader:
                 raise AnsibleParserError("The requested action was not found in configured module paths. "
-                        "Additionally, core modules are missing. If this is a checkout, "
-                        "run 'git pull --rebase' to correct this problem.",
-                        obj=self._task_ds)
+                                         "Additionally, core modules are missing. If this is a checkout, "
+                                         "run 'git pull --rebase' to correct this problem.",
+                                         obj=self._task_ds)
 
             else:
-                raise AnsibleParserError("no action detected in task. This often indicates a misspelled module name, or incorrect module path.", obj=self._task_ds)
+                raise AnsibleParserError("no action detected in task. This often indicates a misspelled module name, or incorrect module path.",
+                                         obj=self._task_ds)
         elif args.get('_raw_params', '') != '' and action not in RAW_PARAM_MODULES:
             templar = Templar(loader=None)
             raw_params = args.pop('_raw_params')
             if templar._contains_vars(raw_params):
                 args['_variable_params'] = raw_params
             else:
-                raise AnsibleParserError("this task '%s' has extra params, which is only allowed in the following modules: %s" % (action, ", ".join(RAW_PARAM_MODULES)), obj=self._task_ds)
+                raise AnsibleParserError("this task '%s' has extra params, which is only allowed in the following modules: %s" % (action,
+                                                                                                                                  ", ".join(RAW_PARAM_MODULES)),
+                                         obj=self._task_ds)
 
         # shell modules require special handling
         (action, args) = self._handle_shell_weirdness(action, args)
