@@ -20,9 +20,9 @@ __metaclass__ = type
 
 import os
 
-from ansible.constants import mk_boolean as boolean
 from ansible.errors import AnsibleError
 from ansible.module_utils._text import to_native
+from ansible.module_utils.parsing.convert_bool import boolean
 from ansible.plugins.action import ActionBase
 
 
@@ -34,8 +34,8 @@ class ActionModule(ActionBase):
 
         result = super(ActionModule, self).run(tmp, task_vars)
 
-        src        = self._task.args.get('src', None)
-        remote_src = boolean(self._task.args.get('remote_src', 'no'))
+        src = self._task.args.get('src', None)
+        remote_src = boolean(self._task.args.get('remote_src', 'no'), strict=False)
 
         if src is None:
             result['failed'] = True
@@ -61,7 +61,7 @@ class ActionModule(ActionBase):
         tmp_src = self._connection._shell.join_path(tmp, os.path.basename(src))
         self._transfer_file(src, tmp_src)
 
-        self._fixup_perms2((tmp, tmp_src) )
+        self._fixup_perms2((tmp, tmp_src))
 
         new_module_args = self._task.args.copy()
         new_module_args.update(
